package capsule;

import node.*;
import utils.*;

import clientserver.NumericResults;
import clientserver.Result;
import node.services.*;

import java.io.*;
import java.util.*;

public class ActiveBWbottleneckAgent extends HashAgent {
    protected static int n=2; // That many measurements are compared at a time (should be > 0)(needs n+1 matching packets).
    protected static long timeout=1000000; // Max time that a series should take (mic s).

    protected byte[] myMatch ={-18, 46, -6, -125, 
-110, -26, -6, -96, 
16, -88, -114, -19, 
92, 114, -111, 61};

    protected long first; // Arrival time of the first matching packet.
    protected long last; // Arrival time of the last matching packet of a serie.

    protected long bytes=0; // Tot number of bytes of the serie.
    protected boolean activated=false; // Are we amidst measuring a series.
    protected int cntr=0;
    protected double bw;

    public void handlePacket (IPPacket p) {
	if(udpPayloadMatch(p)) {
	    //System.out.println("+++MATCH");
	    if(!activated) {
		activated=true;
	    } 
	    nextPacket(p);
	}else  { // no match
	    //System.out.println("NO MATCH----");
// 	    PacketPrinter pr=new PacketPrinter();
// 	    System.out.println(pr.data(p));
// 	    System.out.println("\n----");
	    if(activated) {
		System.err.println("Intervenig packet: "+headerInfo(p));
		// Alarm; if serie ends regularly, the serie is deactivated there..
		activated=false;
	    }
	}
    }

    protected void nextPacket(IPPacket p) {
	if(cntr==0) // first of a serie
	    first=p.timeStamp;
	else {
	    last=p.timeStamp;
	    if(last-first>timeout) { // Timeout?
		// treat the packet as the beginning of a new series.
		first=p.timeStamp;
		System.out.println("UDP bottleneck test loss: "+(n+1-cntr));// Alarm?
		resetCntrs();
		cntr=0; // finally, one packet is counted.
	    } else { //Regular accumulation.
		bytes+=p.len();
		if(cntr>=n) { // End of the serie?
		    bw=((double) bytes)/(last-first);
		    sendResults();
		    resetCntrs();
		}
	    }

	}
	cntr++;
    }

    protected void resetCntrs() {
	cntr=-1;
	activated=false;
	bytes=0;
    }

    public  void  initHash() {
	// Init the Hash IVs here.
	from=28; // udp payload.
	match= myMatch;
	matchLen=128; // This many bits are compared.
	hashFct=new EasyHash();//new MD5hash(); HeaderMatchHash;
    }


    public void handleService (Service s ){}

    public void emergencyHandle(IPPacket p) {}

    public boolean init(Vector initVector) {
	boolean ret=super.init(initVector);
	//	changeChannel("balu.unibe.ch", ProtoConsts.CALLBACK_PORT,false,false);
	return ret;
    } 

    public void stop(){}

//     protected void sendResults() {
// 	NumericResults nr=new NumericResults(last,bw); // Send the bottleneck bw with the appropriate time.
// 	Vector res=new Vector(1);
// 	res.addElement(nr); 
// 	nr.results=res; // For saving in the CallBackDisplayer add the same element as 'results'.
// 	sendResults(nr); 
//     }

    protected void sendResults() {
	Result res=new Result(((new Long(last).toString()))+"\t"+(new Double(bw)).toString()+" MBps.",false);
	sendResults(res);
    }

    protected StringBuffer headerInfo(IPPacket p) {
	StringBuffer buf=new StringBuffer();
    	buf.append('\n');
	buf.append(p.timeStamp);
	buf.append(" Packet ");
	buf.append("\n\t");
	buf.append(" from: ");
	buf.append(AddressCalculator.toString(p.source()));
	buf.append(" to: ");
	buf.append(AddressCalculator.toString(p.destination()));
	buf.append("\n\t");
	buf.append(" proto: ");
	buf.append(p.proto());
	buf.append(" len: ");
	buf.append(p.len());
	buf.append('\n');
	return buf;
    }
}
