package capsule;

import stat.*;
import node.*;
import utils.*;

public class IPConsistencyMeter extends Meter {
    // Used to validate ip header data

    protected boolean print=false; // For debugging, set this to true -> output at the server side;
    protected StringBuffer errors=new StringBuffer(32);

    protected boolean error= false; // subclasses may set this to true
    protected long oldTimeStamp=0; // to check order to timestamps.

    public IPConsistencyMeter(MeterAgent rl) {
	initAgent(rl);
    }

    public void packetArrival(IPPacket p) {
	// Do all kind of format checks on the IP packet.
	short cksum=calcChksum(p);
	if (cksum != -1) { // -1 is 2-complement of all ones.
	    errors.append("\nWrong ip header checksum: "+cksum);
	    error=true;
	}
	// timestap check
	if(p.version() != 4) {
	    errors.append("\nWarning: unknown IP version: "+p.version());
	    error=true;
	}
	if(p.len()<20 || p.len()>=65536){
	    errors.append("\nIllegal packet length: "+p.len());
	    error=true;
	}

	if(p.bytes.length<20 ) {
	    errors.append("\nNot even complete header, only: "+p.bytes.length);
	    error=true;
	}

	if (p.bytes.length != p.len()) {
	    //System.out.println("Warning: too much/little data received: " +p.bytes.length);
	    //dont print packet in this case
	}

	if(p.optLen() < 0) {// standard header is 5 words a 32 bits. 
	    errors.append("\nIP header Header len is too short: "+p.optLen());
	    error=true;
	}

	long ts=p.timeStamp;
	if(ts<oldTimeStamp) {
	    errors.append("\nTimestamps out of order: old: "+oldTimeStamp+" new: "+ts);
	    error=true;
	}
	oldTimeStamp=ts;

	if(error) {
	    if(print) {
		System.out.println(errors);
	    }
	    storeMeasurement(1); // Counts the erroneous packets.
	} else 
	    storeMeasurement(0);
	error=false;
    }

    public String getErrors() {
	errors.insert(0,"\n\nErrors in packet nr: "+cntr);
	return new String(errors);
    }

    protected short calcChksum(IPPacket p) {
	long over,l=0;
	int hl=p.headerLen()*4;
	int test=0;
	for(int i=0; i< hl;) {	
	    l+=p.toPositiveNumber(i, 2);
	    i+=2;
	}
	over=l>>16;
	while(over>0) {
	    l=(l & 0xffff)+over;
	    over=l>>16;
	}
	return (short) l;
    }

}
