package capsule;

import node.*;
import clientserver.NumericResults;
import clientserver.Result;
import node.services.*;

import java.io.*;
import java.util.*;

public class PassiveBWbottleneckAgent extends MyAgent {
    protected static int n=2; // That many measurements are compared at a time (should be > 0).
    protected static double tolerance=0.1; // Max candidates may be that far appart (>0).

    protected double max=0; // The currrently measured bottleneck BW (in Bytes per microsecs).
    protected double curMax=0; // The max in the candidate array;
    protected double[] candidates=new double[n];
    protected int cur=0; // pointer to the current slot in the candidates array.
    protected long last; // Arrival time of the last packet.

    protected long equal=0; // To keep track of how often the bottleneck was hit.

    public void handlePacket (IPPacket p) {
	long t=p.timeStamp;
	long d=t-last;
	last=t;
	double bw=((double) p.len())/d;
	//System.out.print("|BW: "+bw);
	addCandidate(bw);
    }

    public void handleService (Service s ){}

    public void emergencyHandle(IPPacket p) {}

    public boolean init(Vector initVector) {
	for(int i=0; i<n; i++) {
	    candidates[i]=0;
	}
	return super.init(initVector);
    } 
    public void stop(){}

    protected void addCandidate(double bw) {
	candidates[cur++]=bw;
	if (cur==n) cur=0;
	setCurMax();
	if(curMax>max) {
	    if(curMax==max) equal++;
	    else { // A real interesting candidate.
		if(candidatesAreStable()) {
		    max=curMax;
		    sendResults();
		}
	    }
	}
    }

    protected void setCurMax() {  // Could be optimized, but is not worth it given small n.
	double bw;
	curMax=0;
	for(int i=0; i<n; i++) {
	    bw=candidates[i];
	    if (bw>curMax) curMax=bw;
	}
    }

    protected boolean candidatesAreStable() {
	// Compare the standard deviation of the candidates.
	if(n<2) return true;
	double avrg;
	double sum=0;
	for(int i=0; i<n; i++) {
	    sum+=candidates[i];
	}
	avrg=sum/n;
	double stddev;
	double d;
	sum=0;
	for(int i=0; i<n; i++) {
	    d=avrg-candidates[i];
	    sum+=d*d;
	}
	sum=sum/(n-1);
	stddev=Math.sqrt(sum);
	stddev=stddev/avrg;
	//System.out.print("| Dev% "+stddev);
	return (stddev<tolerance);
	 
    }

//     protected void sendResults() {
// 	NumericResults nr=new NumericResults(last,max); // Send the bottleneck bw with (almost) the appropriate time.
// 	Vector res=new Vector(1);
// 	res.addElement(nr); 
// 	nr.results=res; // For saving in the CallBackDisplayer add the same element as 'results'.
// 	sendResults(nr); 
//     }

    protected void sendResults() {
	Result res=new Result(((new Long(last).toString()))+"\t"+(new Double(max)).toString(),false);
	sendResults(res);
    }


}
