package capsule;

import stat.*;
import node.*;

import java.util.*;

public class RunLengthMeter extends PayloadStatMeter {
    // Implements the simplified run-test for increasing runs.
    // In order to use the chi-square statistics, the byte between two runs
    // is ignored, thus the runs are not dependent.
    protected int singleton=0;
    protected short pos=0; 
    protected int old=0;
    protected boolean start=true;
    protected int runLen=0; 


    public RunLengthMeter(MeterAgent rc, boolean sig) {
	significant=sig;
	initChi();
	initAgent(rc); 
    }

    public void reset() {
	super.reset();
	pos=0;
	runLen=0;
	start=true;
    }
    
    protected  void initChi() {
	// The categories are number of run-lengts from 1-6. All runs
	// longer than 6 are also classified in category 6.
	RunDistribution dist= new RunDistribution(6);
	chi=new X2(dist, 5);
    }

    protected void treatByte(byte b) {
	if (pos<4) {
	    singleton= singleton << 8;
	    singleton+=b;
	    pos++;
	}
	if (pos==4) {
	    treatNewInt(singleton);
	    singleton=0;
	    pos=0;
	}
    }

    protected void treatNewInt(int s) {
	//System.out.println(s);
	if (start) {
	    start=false;
	    old=s;
	    // Ignore the first measurement of each run (makes the run-length independent).
	} else {
	    if (s<old) runEnd(s);
	    else runContd(s);
	}

    }

    protected void runContd(int s) {
	runLen++;
	old=s;
    }

    protected void runEnd(int s) {
	if (runLen>5) runLen=5; // All long runs are in the uppermost category.
	addNewMeasurement(runLen);
	runLen=0;
	start=true;
    }

//     public static void main(String args[]) {
// 	Random rnd=new Random();
// 	RunLengthMeter m=new RunLengthMeter(1000,true);
// 	long stop,start=System.currentTimeMillis();
// 	for (int i=0; i< 1000000; i++) {
// 	    m.treatByte((byte) rnd.nextInt());
// 	}
// 	stop=System.currentTimeMillis();
// 	System.out.println(m);
// 	System.out.println("Executiontime: "+(stop-start)); // ~8sec -> 1Mbps
	
	
//     }

}
