package clientserver;
import java.io.*;

public class ByteArray implements Serializable {
    //Transport container for the byte code
    public byte[] bytes=null; // can be null, e.g. if io-errors occur.
    
    public ByteArray(byte[] b) {
	bytes=b;
    }

    public ByteArray(ByteArray b) {
	bytes=b.bytes;
    }

    

    public ByteArray(String name) {
	readByteCodeFromFile(name);
    }

    public int length() {
	return bytes.length;
    }

    public ByteArrayInputStream getInputStream() {
	// for convenient access to the bytes.
	if(bytes==null) {
	    System.err.println("Warning! ByteArray is empty!");
	    return new ByteArrayInputStream(new byte[0]);
	}
	return new ByteArrayInputStream(bytes);
    }

    public void readByteCodeFromFile(String name) {
	// sets the 'bytes' field.
	try {
	    FileInputStream fs=new FileInputStream(name);
	    readByteCodeFromStream(fs);
	} catch (IOException e) {
	    bytes=null;
	    System.err.println(e);
	}
    }

    public void readByteCodeFromStream(InputStream is) {
	// sets the 'bytes' field.
	bytes=new byte[16];
	int bfr=128;
	int bytesread=0;
	int offset=0;
	try{   
	    while (bytesread>=0) {
		bytesread=is.read(bytes, offset, bfr);
		if(bytesread== -1) break;
		offset += bytesread;
		if(offset==bytes.length) { // Grow the array
		    byte temp[] = new byte[offset*2];
		    System.arraycopy(bytes,0,temp,0,offset);
		    bytes=temp;
		} else if (offset> bytes.length) {
		    System.err.println(is+": error reading data into array!");
		    bytes=null;
		    return;
		}
	    }
	    
	    if (offset < bytes.length) { // shrink the array.
		byte temp[]=new byte[offset];
		System.arraycopy(bytes,0,temp,0,offset);
		bytes=temp;
	    }
	} catch (IOException e) {
	    closeStream(is);
	    bytes=null;
	    System.err.println(e);
	}
	closeStream(is);
    }


    protected void closeStream(InputStream is) {
	// This is important to remove unnecesary files when is is a OneTimeInputStream.
	try{is.close();} catch(IOException e){};
    }
}
