package clientserver;

import java.util.*;
import homeApplication.*;
import netgui.*;

public class CallBackProtocol extends MessageProtocol {
    // Steers the customer part of the interaction protocol when an agent
    // has opened a callback connection.
    // Encapsulates the feedback of one agent.

    protected CallBackDisplay display;
    protected ValueDisplayer value; // to forward the results directly.
    protected boolean receiving=true;
    ExecProtoDisplay alarmDisplay=null;

    public CallBackProtocol(Client c, CallBackDisplay d) {
	init(c);
	display=d;
	posr=client.getProtocolObjectSenderReceiver();
    }

    public void stop() {
	receiving=false;
	client.closeConnection();
    }

    public synchronized void sendMessage(Message m, String sid, byte enc, String passf) {
	System.err.println("CallBackProtocol does not send something!");
    }

    public byte messageType() {return ProtoConsts.CALLBACK;}


    protected synchronized void runProtocol() {
	CallBack cb=receiveCallBack();
	if(cb==null) {
	    System.err.println("Received empty callback!");
	} else {
	    System.err.println("Received: "+cb);
	    value=display.handle(cb, this); // registers to the display. 
	    receiveResults();
	}
    }

    protected CallBack receiveCallBack() {
	CallBack ack=null;
	DecodingResult dr=posr.receive(passPhrase);
	ProtocolObject po=dr.protocolObject();
	if(po==null) {
	    return ack;
	}
	checkAuthenticity(dr);
	if (po != null) {
	    if (po.messageType==ProtoConsts.CALLBACK) {
		ack=(CallBack) Message.readObjectFrom(po);
	    }
	}
	return ack;
    }

 
    protected void receiveResults() {
	DecodingResult dr;
	while (receiving) { // Reads replies as long as they keep coming.
	    dr=posr.receive(passPhrase);
	    ProtocolObject po=null;
	    if(dr != null)
		po=dr.protocolObject();
	    if(po==null) {
		value.setError("Connection broken!");
		return;
	    }
	    checkAuthenticity(dr);

	    if (po.messageType != ProtoConsts.RESULT) {
		System.err.println("Received agent call back with  wrong message type: "+po.messageType);
		value.setError("Agent reply type is faulty!");
		return; // the client will then close the connection.
	    } else {
		Message m=(Message) Message.readObjectFrom(po);
		NumericResults r=null;
		if (m instanceof Result) {// We have to go through this because the last message might
		    Result res=(Result) m;// be automatically generated, and of class Result.
		    if(res.isLast()) {
			r=new NumericResults(res); // translation to a 'last' num. result.
		    }		    
		    if ((m instanceof NumericResults)) {
			r=(NumericResults) m;
		    }
		    if(r ==null) {
			value.setError("Received strange result.");
			return;
		    } else {
			value.setResult(r);
			if(r.alarm) { // If its an alarm we pop up a window to display it.
			    handleAlarm( r,  po);

			}
			if(r.isLast()){
			    receiving=false; // No further result is to be expected.
			}
		    }
		    display.repaintCanvas(); // Make sure the new result is drawn
		} else {
		    value.setError("Received malformed result: "+m);
		    System.err.println("Error: expecting an object of class NumericResults, but got: "+m);
		    return;
		}

	    }
	} 
    }

    protected void handleAlarm(NumericResults r, ProtocolObject po) {
	if(alarmDisplay==null) {
	    alarmDisplay=new ExecProtoDisplay(null);
	    alarmDisplay.show();
	}
	alarmDisplay.setStatus("ALARM at: "+po.senderID);
	alarmDisplay.handle(r);
    }

}
