package clientserver;

import java.util.*;
import homeApplication.*;

public class ExecRequestProtocol extends MessageProtocol {
    // Steers the client part of the interaction protocol to execute agents.

    //protected long strt; // Performance measuremetns

    public ExecRequestProtocol(Client c) {
	init(c);
	display=new ExecProtoDisplay(this); 
    }

    public void stop() {
	client.closeConnection();
    }

    public synchronized void sendMessage(Message m, String sid, byte enc, String passf) {
	if(!(m instanceof ExecutionRequest)) {
	    System.err.println("Error: expecting a ExecutionRequest.");return;
	}
	initPGPExpectations(passf,enc); // The answer should use the same passphrase and
	// encoding.
	sendExecRequest((ExecutionRequest) m, sid, enc);
    }

    public byte messageType() {return ProtoConsts.EXEC_REQ;}


    protected synchronized void sendExecRequest(ExecutionRequest er, String id, byte enc) {
	// The flow of the protocol. Mainly sending the request and then 
	// receiving answers as long as they keep coming.
	ProtocolObject po= createRequest(er, id, enc); 
	po.forwarding=er.forwarding; // This is a hack because we have those both fields.
	display.show(); // activate the gui window.
	//strt=System.currentTimeMillis(); // perf. measurement
	if(posr.send(po,passPhrase)) { // The protocol sequence for agent execution;
	    Acknowledgement ack=receiveAck();

	    if (ack.isOK()) { 
		display("Node acknowledges the agent.");
		//System.out.println("Ack is OK!");
		if(po.forwarding instanceof Forwarding) { // Forwarded agents cannot keep up the homechannel.
		    display("Agent is disconnected for forwarding."); // The server will close the connection.
		    try {Thread.sleep(500);}catch(InterruptedException e){};
		    display.stop();
		} else 
		    receiveResults(); // Receive results as long as they are coming.
	    } else {
		//System.out.println("Ack NOT OK! Reason: ");
		//System.out.print(((Nacknowledgement) ack).reason()); 
		display("NACK: "+((Nacknowledgement) ack).reason());
	    }
	} else {
	    display("Could't send/encode the message! \nWrong IDs or passphrase?");
	}
    }

    protected Acknowledgement receiveAck() {
	Acknowledgement ack=new Nacknowledgement("Could not receive Ack.");
	DecodingResult dr=posr.receive(passPhrase);
	//System.out.println("Receive ack: "+dr);
	ProtocolObject po=getProtocolObject(dr);
	if(po==null) {
	    return ack;
	}
	checkAuthenticity(dr);
	// Add auth. checks here!
	if (po != null) {
	    if (po.messageType==ProtoConsts.EXEC_ACK) {
		ack=(Acknowledgement) Message.readObjectFrom(po);
	    }
	}
	return ack;
    }

    protected void receiveResults() {
	while (true) { // Reads replies as long as they keep coming.
	    DecodingResult dr=posr.receive(passPhrase);
	    //System.out.println("Result arrival: "+(System.currentTimeMillis()));

	    ProtocolObject po=getProtocolObject(dr);

	    //long fini=System.currentTimeMillis();
	    //System.out.println("Response time (result): "+(fini-strt));

	    if(po==null) {
		return;
	    }
	    checkAuthenticity(dr);

	    if (po.messageType != ProtoConsts.RESULT) {
		System.err.println("Received agent reply with wrong message type: "+po.messageType);
		display("Agent reply type is faulty!");
		return; // the client will then close the connection.
	    } else {
		Message m=(Message) Message.readObjectFrom(po);
		display.handle(m);
		if (!(m instanceof Result)) {
		    System.err.println("Error: expecting an object of class Result.");
		    return;
		}
		Result r=(Result) m;
		if(r.isLast()){
		    display("Agent has finished.");
		     break; // No further result is to be expected.
		}
	    }
	} // The loop has several exit points, this is spaghetti...
    }


    protected void display(String s) {
	((ExecProtoDisplay) display).setStatus(s);
	display.repaint();
	//Thread.yield();
    }

}
