package clientserver;

import java.util.*;
import homeApplication.*;

public abstract class MessageProtocol {
    // Steers the client part of the interaction protocol for a given message exchange.

    protected ProtocolObjectSenderReceiver posr;
    
    protected Client client; // The client who created me (for stopping the connection).
    protected ProtoDisplay display;

    protected boolean mustBeAuthentic; //

   protected String passPhrase;// Store the last pph in order to use it when getting an answer
    // from the server.

    protected void init(Client c) {
	client=c;
    }

     void initPGPExpectations(String passf, byte requestEncoding ) { 
	// Should be called when sending a request, or when it is otherwise
	 // known how answers should be encoded.
	passPhrase=passf; // Keep it for decoding the server answer.
 	if (requestEncoding==ProtoConsts.FULL_PGP_v263i)
 	    mustBeAuthentic=true; // The answers must also be authenticated if the request is.
	//	mustBeAuthentic=true;  // hard-code authenticity check eg for testing.
    }

    protected ProtocolObject createRequest(Message m, String id, byte enc) {
	// Creates a protocl object.
	posr=client.getProtocolObjectSenderReceiver(); // Sets the posr as a side-effect.
	return new ProtocolObject(id, enc, messageType(),  m);
    }

    public abstract synchronized void sendMessage(Message m, String id, byte enc, String passf);

    public abstract byte messageType(); // The value in ProtoConsts that codes the
    // type of message this protocol does handle.

    public void stop() {
	client.closeConnection();
    }

    protected ProtocolObject getProtocolObject(DecodingResult dr) {
	// Extracts the PO if possible. If not, informs via GUI.
	ProtocolObject ret=null;
	if(dr==null) {
	    NotifyUser nu=new NotifyUser(display, "Connection interrupted!");
	    nu.setVisible(true);
	} else ret=dr.protocolObject();
	return ret;
    }

    protected void checkAuthenticity(DecodingResult dr) {
	// If the protocol requests authenticity (mustBeAuthentic), display
	// if something is wrong with the signature
	String text=null;
	if(mustBeAuthentic) {
	    if(dr.isSigned()) {
		if(!dr.signedBy().equals(dr.protocolObject().senderID))
		    text="Warning! Reply message is signed by\n"+dr.protocolObject().senderID+" instead of\n"+dr.signedBy();
	    }  else
		text="Warning! Reply message is unsigned!";
	    if(text != null) {
		NotifyUser nu=new NotifyUser(display, text);
		nu.setVisible(true);
	    }
	}

    }



}
