package filter;

import utils.*;
import java.util.*;


public class  FilterEntryParser {
    // Can parse different string representations 
    // of filter entries (ListOfRanges or Length).

    public final static String WILDCARD="*";

    protected StringTokenizer whiteSpaceParser;
    //protected StringTokenizer minusParser;

    public String wildcard() {
	return WILDCARD;
    }
    
    public Length parseLength(String s) {
	if(isWildCard(s)) return new Length(-1); // negative numbers encode wildcards.
	try {
	    return new Length(Integer.parseInt(s));
        } catch(NumberFormatException e) { 
	    return new Length(0);
	}
    }

    public ListOfRanges parseAddressRanges(String s) {
	// Interpretes the address as a range
	if(isWildCard(s)) return new All();
	whiteSpaceParser=new StringTokenizer(s,"\n\t ");
	String token;
	ListOfRanges ret=new ListOfRanges();
	Range r;
	while (whiteSpaceParser.hasMoreTokens()) {
	    token=whiteSpaceParser.nextToken();
	    r=AddressCalculator.parseRange(token); // safety check?
	    ret.addRange(r); 
	}
	return ret;
    }

    public ListOfRanges simplifiedParsing(String s) {
	// Assumes a white space seperated list of single number values.
	if(isWildCard(s)) return new All();
	whiteSpaceParser=new StringTokenizer(s);
	String token;
	Vector v=new Vector();
	int i;
	while(whiteSpaceParser.hasMoreTokens()) {
	    token=whiteSpaceParser.nextToken();
	    try {
		i=Integer.parseInt(token);
		if (i>=0) // safety check
		    v.addElement(new Range(i));
		
	    } catch(Exception e) {}// just remove the entries that do not parse
	}
	return new ListOfRanges(v);
    }

    public String printAddressRanges(ListOfRanges ranges) {
	// Prints the list of address ranges the same way as a user
	// should enter it.
	Range r;
	StringBuffer buf=new StringBuffer();
	for(Enumeration e=ranges.list().elements(); e.hasMoreElements(); ) {
	    r=(Range) e.nextElement();
	    buf.append(AddressCalculator.toDotString(r));
	    buf.append(' ');
	}
	return new String(buf);
    }

    public String printSimplifiedRanges(ListOfRanges ranges) {
	//  Prints the list of ranges as a list of single values (using
	// the lower bound of the range.
	if(ranges instanceof All) {return wildcard();}
	Range r;
	StringBuffer buf=new StringBuffer();
	for(Enumeration e=ranges.list().elements(); e.hasMoreElements(); ) {
	    r=(Range) e.nextElement();
	    buf.append(r.low());
	    buf.append(' ');
	}
	return new String(buf);
    }

    public boolean isWildCard(String s) {
	return s.equals(wildcard());
	// prototypical (no whitespace consumption)
    }


}
