package filter;

import utils.*;
import java.io.*;
import java.util.*;


public class  ListOfRanges implements Serializable {
    // Describes a set of integers as a sorted list of ranges (from..to).
    protected Vector list=new Vector();

    public ListOfRanges() {}; // Empty list

    public ListOfRanges(int single) {
	this(single,single);
    }

    public ListOfRanges(int low, int up) {
	this(new Range(low, up));
    }
    
    public ListOfRanges(Range single) {
	list.addElement(single);
    }

    public ListOfRanges(Vector is) {
        // Warning! Ranges in vector is should be sorted (better use the addRange() method)!
	initWith(is);
    }

    protected void initWith(Vector v) {
	list=v;
    }

    Vector list() {
	// for FilterEntryParser 
 	return list;
     }

    Range first() {
	// convenient accessor;
	return (Range) list.firstElement();
    }

    public boolean match(int i) {
	// if something matches this list of ranges (used for filtering)
	Range r;
	for(Enumeration e=list.elements(); e.hasMoreElements(); ) {
	    r= (Range) e.nextElement();
	    if(r.match(i)) return true;
	}
	return false;
    }

    // --- Set operations

    public boolean isEmpty() {
	return (list.size()<1);
    }

    public ListOfRanges and(ListOfRanges other) {
	Vector res=new Vector();
	Range r1=null,r2=null;
	Range cut;
	Enumeration e1=other.list.elements();
	Enumeration e2=list.elements();
	if (e1.hasMoreElements() && e2.hasMoreElements()) {
	    r1=(Range) e1.nextElement(); r2=(Range) e2.nextElement(); 
	    while (true) {
		cut=r1.and(r2);
		if(!cut.isEmpty()) res.addElement(cut);
		if(r1.up()<r2.up())
		    if(e1.hasMoreElements())
			r1=(Range) e1.nextElement();
		    else break;
		else 
		    if(e2.hasMoreElements())
			r2=(Range) e2.nextElement(); 
		    else break;
	    }
	}
	return new ListOfRanges(res);
    }


    public ListOfRanges or(ListOfRanges other) {
	// Logical or
	return other.internalOr(this); // to get the All case right
    }

    protected ListOfRanges internalOr(ListOfRanges other) {
	Vector backup=(Vector) list.clone(); // preserves my state
	ListOfRanges ret;
	Range r;
	for(Enumeration e=other.list.elements(); e.hasMoreElements(); ) {
	    r= (Range) e.nextElement();
	    addRange(r);
	}
	ret=new ListOfRanges(list);
	list=backup;
	return ret;
    }

    public void addRange(Range r) {
	// Adds a range in the sorted order.
	if(r.isEmpty()) return;
	Range nr;
	boolean added=false;
	int i=0;
	for(Enumeration e=list.elements(); e.hasMoreElements(); i++ ) {
	    nr= (Range) e.nextElement();
	    if (r.smaller(nr)) {
		list.insertElementAt(r,i); added=true; break;
	    }
	    if (!nr.smaller(r)) { // an overlapping situation
		int k=i;
		Range last,first=nr;
		while(nr.touching(r) && e.hasMoreElements()) {
		    nr= (Range) e.nextElement();
		    i++;
		}
		if(!nr.touching(r)) {
		    i--;
		}
		last=(Range) list.elementAt(i);
		mergeRanges(r,first,last,k,i); added=true; break;
	    }
	}
	if(!added) { // the new range is at the end of the list.
	    list.addElement(r);
	}
    }

    protected void mergeRanges(Range r,Range first,Range last,int k,int i) {
	// ranges k to i over lap with r. create one big range
	//System.out.println("Merging: "+list+ "\nfirst " +first+" last: "+last+" i "+i+" k "+k);
	int min=r.low();
	int max=r.up();
	if (first.low()< min) min=first.low();
	if (last.up()> max) max=last.up();
	for(int j=k; j<=i; j++) { // remove the seperate parts
	    list.removeElementAt(k);
	}
	list.insertElementAt(new Range(min,max),k);

    }

//     public boolean isSubsetOf(Wildcard l) {
// 	for (int i=0; i<list.length;i++) {
// 	    if (!l.match(list[i])) return false;
// 	}
// 	return true;
//     }


//     public boolean match(int erep) {
// 	for (int i=0; i<list.length;i++) {
// 	    if (list[i]==erep) return true;
// 	}
// 	return false;
//     }


//     public Wildcard and(Wildcard e) {
// 	// works for all wildcards, but is not optimal.
// 	int tmp[]=new int[list.length];
// 	int cnt=0;
// 	int h=0;
// 	for (int i=0; i<list.length;i++) {
// 	    h=list[i];
// 	    if (e.match(h)) {
// 		tmp[cnt]=h;
// 		cnt++;
// 	    }
// 	}
// 	int[] res=copy(tmp, cnt);
// 	return new List(res);
//     }

//     public Wildcard and(All e) {return this;}


    // --- Output to String

     public String toString() {
	 return super.toString()+list.toString();
     }
// 	StringBuffer buf=new StringBuffer();
// 	//buf.append(super.toString());
// 	buf.append("List: ");
// 	for (int i=0; i<list.length;i++) {
// 	    buf.append(list[i]);
// 	    buf.append(' ');
// 	}
// 	return new String(buf);
//     }

    public String tcpDumpProtoSpec() {
	// Creates a tcpdump selector that matches the given protocol numbers.
	// Note that for simplicity, not complete ranges are matched but only the
	// lower bound of all ranges.
	StringBuffer buf=new StringBuffer();
	orLoop(buf, "ip[9]==", false);
	return new String(buf);
    }

    public String tcpDumpPortSpec() {
	// Creates a tcpdump selector that matches the given Port numbers of either
	// source or destination (encoded in 'direction').
	// Note that for simplicity, not complete ranges are matched but only the
	// lower bound of all ranges.    
	StringBuffer buf=new StringBuffer();
	orLoop(buf, "port ", false);
	return new String(buf);
    }


    public String tcpDumpAddrSpec(String direction) {
	if (this instanceof All) return "";
	StringBuffer buf=new StringBuffer();
	orLoop(buf, direction, true);
	return new String(buf);
    }

    protected String orLoop(StringBuffer buf, String keyword, boolean netAdd) {
	// Combining all list entries with logical or's
	Range nr;
	buf.append('(');
	for(Enumeration e=list.elements(); e.hasMoreElements();) {
	    nr= (Range) e.nextElement();	
	    buf.append('(');
	    if (netAdd)	 buf.append(nr.tcpDumpAddrSpec(keyword));
	    else {
		buf.append(keyword);
		buf.append(nr.low()); // simplification: use the lower range bound.
	    }
	    buf.append(')');
	    if (e.hasMoreElements()) 
		buf.append(" || ");

	}
	buf.append(")");
	return new String(buf);
    }


    public static void main(String[] args) {
// 	ListOfRanges lr=new ListOfRanges(new Range(10));
// 	lr.addRange(new Range(2,3));
// 	lr.addRange(new Range(1));
// 	lr.addRange(new Range(8,9));
// 	lr.addRange(new Range(5,6));

// 	ListOfRanges lr2=new ListOfRanges(new Range(3));
// 	lr2.addRange(new Range(8,9));
// 	lr2.addRange(new Range(22,22));

//  	System.out.println(lr);
//  	System.out.println(lr2.and(lr));
//  	System.out.println(lr.and(lr2));

// 	ListOfRanges lr3=new ListOfRanges(new Vector());
	

// 	lr2=new All();
// 	System.out.println("\n\n");
// 	System.out.println(lr.and(lr2));
// 	System.out.println(lr3.or(lr2));
// 	System.out.println(lr2.or(lr3));

        Range rng=utils.AddressCalculator.parseRange("10");
        ListOfRanges lr=new ListOfRanges(rng);
        lr.addRange(utils.AddressCalculator.parseRange("172"));
        lr.addRange(utils.AddressCalculator.parseRange("130.92"));
        ListOfRanges req=new ListOfRanges(utils.AddressCalculator.parseRange("130.92.65"));
        req.addRange(utils.AddressCalculator.parseRange("130.94.65.15"));

        //	ListOfRanges lr=new ListOfRanges(new Range(-2107899904, -2107834369));
        //	lr.addRange(new Range(  -2107817472 , -2107817217));
        //System.out.println(lr.tcpDumpAddrSpec("src"));
        //System.out.println(lr.tcpDumpProtoSpec());
        //System.out.println(lr.tcpDumpPortSpec());
        System.out.println((lr.and(req)).tcpDumpAddrSpec("test"));


       

    }

}
