package homeApplication;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import netgui.*;
import clientserver.*;
import topology.*;
import utils.AddressCalculator;

public class CallBackDisplay extends ProtoDisplay implements Runnable, ItemListener {
    // Displays what forwarded agents send back via a call back channel.
    // Discovery through call backs.
    // This class relays the incoming data to the gui. It manages (starts/stops) both parts.

    public static boolean record; // To globally turn on/off recording of result values (see netgui.ValueDisplayer).
    protected static String savePath="/home/mguenter/rvs/Agents/SeparateClientTest/Save/";

    protected NetCanvas canvas=new NetCanvas();
    protected Button stop=new Button("Stop");
    protected Checkbox recordCB=new Checkbox("Record",false);
    protected Button save=new Button("Save selected");
    protected TextField status=new TextField("Status: ",20);
    protected Panel control=new Panel(); 

    protected Server server; // To receive the callbacks.
    protected Vector cbChannels=new Vector(); // a Vector of CallBackProtocols. Used to close the conns.
    protected Positioner positioner=new Positioner();

    protected ValueDisplayer vdTemplate;

    public CallBackDisplay(ExecutionRequest er, ValueDisplayer vd) {
	vdTemplate=vd;
	initPanel(er);
	initCanvas();
    }

    public NetCanvas getCanvas() {
	return canvas;
    }

    protected void initPanel(ExecutionRequest er) {
	//control.setLayout()
	String agentclass=(String) er.classNames.firstElement();
	String id=(new Integer(er.ID)).toString();
	control.add(new Label("Agent class: "+agentclass));
	control.add(new Label("ID: "+id));
	control.add(stop);
	control.add(recordCB);
	control.add(save);
	control.add(status);
	status.setEditable(false);
	
	stop.addActionListener(this);
	save.addActionListener(this);
	recordCB.addItemListener(this);

	recordCB.setState(true); // Immediately begin recording.
	record=true;
    }


    public void run() {
	// Start the callback server
	server=new Server(ProtoConsts.CALLBACK_PORT, new CallBackHandler(), this);
	server.serverLoop();

    }

    protected void initCanvas() {
	setLayout(new FlowLayout());
	canvas.addMouseListener(canvas);
	canvas.addMouseMotionListener(canvas);
	add(canvas);
	add(control);
    }

    public synchronized void handle(Message m) {
	System.err.println("Old handle");
    };

    public synchronized void register() {

    }

    public synchronized ValueDisplayer handle(CallBack cb, CallBackProtocol proto) {
	// Displays a new router in ev. a new domain. Returns the
	// value displaying object as a short-cut for result notification.
	cbChannels.addElement(proto); // This is the first contact - we need the ref to close the conn later.
	NodeRecord nr=cb.nodeRecord();
	Domain d=canvas.domainOf(nr.organisation);
	if(d==null) {// This domain was not discovered yet:
	    d=createDomain(cb);
	    positioner.put(d, cb.hopCount());
	    canvas.addDomain(d);
	} else {System.out.println("Router in same domain.");}
	Router r=createRouter(d,cb); // Warning: no duplication check.
	positioner.put(r,d); // Places the router on the domain.
	
	adjustNeighborStuff(r,d,cb); //locate connections and customers.
	canvas.addRouter(d,r);
	canvas.repaint();
	return r.valueDisplayer();	       
    };

    protected void adjustNeighborStuff(Router r, Domain d, CallBack cb) {
	// changes positon of the router to face adjacent routers.
	// Hack, hack.
	String nodeId=cb.nodeRecord().key; // The following is quite clumsy...
	NeighborRecord users,nodes,neigh=SDC.neighborLookup.allNeighborsOf(nodeId);

	NodeLookupTable nl=SDC.nodeLookup;
	users=nl.directUsers(neigh);
	nodes=nl.neighborNodes(neigh);
	    
	NodeRecord nr; EndUser eu=null;
	for(Enumeration e=nl.recordsOf(users).elements(); e.hasMoreElements();) {
	    nr=(NodeRecord) e.nextElement(); // Check for neighbor EndUser.
	    // Create a user and place it.
	    eu=canvas.customerAlreadyDisplayed(nr);
	    if(eu == null) { // a new enduser detected.
		// Maybe add a check if this is also the owner of the agent.
		String ip=AddressCalculator.toString(nr.ip);
		eu=new EndUser(canvas, ip , nr.key);
		positioner.put(eu,r);
		canvas.addEndUser(eu);
	    } 
	    r.addBidirectLinkTo(eu);
	}

	Router nero;
	double x=0,y=0;
	int neighs=0;
	for(Enumeration e=nl.recordsOf(nodes).elements(); e.hasMoreElements();) {
	    nr=(NodeRecord) e.nextElement();
	    nero=canvas.alreadyDisplayed(nr,d); // Is there a neighbor router displayed?
	    // note: this will not return neighbors of the same domain.
	    if(nero != null) { // If yes, add the link and use the neighbor for positioning
		// the new router (minimal distance to all neighbors).
		System.out.println("Found drawn Neighbor.");
		Dimension pos=nero.position();
		x+=pos.width; y+= pos.height; neighs++;
		r.addBidirectLinkTo(nero);
	    }
	}
	if(neighs>0) {
	    int nx=(int) x/neighs; int ny=(int) y/neighs; // Central/average pos.
	    positioner.put(r, nx,ny); //Move on that pos (the move method will automatically keep
	    // the router attached to the domain.
	}
	

    }

    public synchronized void repaintCanvas() {
	// to be called when new results arrive.
	canvas.repaint();
    }

    public synchronized void actionPerformed(ActionEvent e) {
	String command=e.getActionCommand(); 
	if(command.equals("Stop")) {
	    stop();
	} else if(command.equals("Save selected")) {
	    String saved=canvas.saveResultsOfSelectedRouter(savePath);
	    if(saved.equals(""))
		status.setText("Select a router.");
	    else
		status.setText("Saved into: "+saved);
	}
    }

    public  void itemStateChanged(ItemEvent e) {
	// Only one checkbox: recordCB
	record=	recordCB.getState();
    }

  
    public synchronized void stop() {
	server.stop();
	CallBackProtocol p;
	for(Enumeration e=cbChannels.elements(); e.hasMoreElements();) {
	    // Close the open connections.
	    p=(CallBackProtocol) e.nextElement();
	    p.stop();
	}
	super.stop();
    }

    protected Domain createDomain(CallBack cb) {
	return new Domain(cb.nodeRecord().organisation, -1);
    }

    protected Router createRouter(Domain d, CallBack cb) {
	NodeRecord nr=cb.nodeRecord();
	String ip=AddressCalculator.toString(nr.ip);
	Router router=new Router(d, canvas, ip , nr.key); 
	router.attach(new ValueDisplayer(vdTemplate)); // use the template in a copy constructor.
	return router;
    }



//     public static void main(String[] args) {
// 	Frame f=new Main();
// 	f.setTitle("Test");
// 	//f.setSize(200,200);
// 	f.pack();
// 	f.show();
//     }
}
