package homeApplication;

/* Written by Manuel Gnter. All rights reserved, no warranties */

import java.awt.*;
import java.io.*;

import node.IPPacket;


public class DataxelCanvas extends Component {
    // This graphical component has a special painter to draw bytes
    // as dataxels (colored, filled boxes).
    // The mapping from bytes to dataxels is done by an object of a subclass of
    // DataxelPainter.

    protected int posX=0, posY=0;
    protected int dataxelSize=10;
    protected int dataxelPerRow=32;
    protected int testsize=1024; // for the test generation. 

    protected DataxelPainter painter; // Is dynamically replaced.

    protected int dataSize;
    protected Dimension mySize;

    //Dimension size=new Dimension(540,400); // Does not seem to work.

    public DataxelCanvas(IPPacket ip) {
	InputStream i=getStream(ip);
	//generateTestStream(); // testing

	painter= new GrayPainter(this, i);

	adjustSize();
    }

    protected void recalcSize() {
	// when the new data size is known.
	int nrOfRows=(8*dataSize)/(painter.BitsPerDataxel()*dataxelPerRow);
	if(dataSize%dataxelPerRow!=0) nrOfRows++;
	mySize=new Dimension(dataxelSize*dataxelPerRow,dataxelSize*nrOfRows);
    }

    public DataxelCanvas(int dSize, int dPerRow, DataxelPainter p, InputStream i) {
	// obsolete
	dataxelSize=dSize;
	dataxelPerRow=dPerRow;
	painter=p;
	if (i==null) {
	    System.err.println("No inputfile provided -> I use a testcase!");
	    i= generateTestStream();
	}
	if (p== null) painter= new GrayPainter(this, i);

	setSize(mySize);
	validate();
    }


    public DataxelPainter getPainter() { return painter;}

    public void setPainter(DataxelPainter p) {
	painter=p;
	repaint();
    }

    public void paint(Graphics g) {
	painter.drawAll(g);
	resetCoordinates();
    }

    public void drawNext(int color, Graphics g) {
	// This method is called back by the DataxelPainter.
	g.setColor(new Color(color));
	g.fillRect(posX,posY,dataxelSize,dataxelSize);
	calcNextPos();
    }
 
    public Dimension getPreferredSize() { 

 	return mySize;
    }


    protected void adjustSize() {
	recalcSize();
	//invalidate();
	//setSize(mySize);
	//validate();
    }

    protected void calcNextPos() {
	posX += dataxelSize;
	if (posX >= dataxelSize*dataxelPerRow) {
	    posX=0;
	    posY += dataxelSize;
	}
    }

    protected void resetCoordinates() {
	posX=0; posY=0;
    }

    protected InputStream getStream(IPPacket ip) {
	byte[] bytes=ip.bytes;
	dataSize=bytes.length;
	return new ByteArrayInputStream(bytes);
    }

    protected InputStream generateTestStream() {
	// If no input stream is provided, we generate a test stream.
	ByteArrayOutputStream s= new ByteArrayOutputStream(testsize);
	for (int i=0; i<testsize; i++) {
	    s.write(i%256-128);
	}
	dataSize=testsize;
	return new ByteArrayInputStream(s.toByteArray());
    }


}
