package homeApplication;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;
import java.io.File;

import clientserver.*;
import utils.*;


public class ExecutionMessageForm extends MessageForm implements ActionListener {
    // The user can fill in form describing where and how to
    // send an execution  message. Default values are displayed.

    protected static String hopByHop="Hop-by-hop"; // To label and map list entries of 'forw'.
    protected static String broadcast="Broadcast"; 

    protected static final String fileExtension=".names";
    protected static final String mandatoryPartOfFilename="Agent"; // This is an enforced naming convention. .. still needed?
    protected static final String path="/home/mguenter/rvs/Agents/SeparateClientTest/";// Basically the classpath of SDC
    protected final static String packageName="capsule"; 

    protected TextField agent;
    protected Label a;
    protected FileDialog fileDialog;
    protected SingleEntryDialog towardsIPdialog;
    protected Button browse=new Button("Browse"); // Lookup an agent name.
    protected Label dummy=new Label("");
    protected ExecutionRequest execReq=null;

    protected ForwardingDescriptor forwarding=null;

    protected static int currentAgentNumber=0; // To set the id of the agent

    public ExecutionMessageForm(Frame parent) {
	setLayout(new GridLayout(7,2,5,5));
	init();
	//init the file dialog stuff:
	fileDialog=new FileDialog(parent, "Agent Bytecode", FileDialog.LOAD );
	towardsIPdialog=new SingleEntryDialog(parent, "Enter target ip: ", true , true);
    }

    public void createFields() {
	super.createFields();
	a=new Label("Agent");
	agent=new TextField(20);
	agent.addActionListener(this);
	browse.addActionListener(this);
	add(a); add(agent);
	add(dummy);  add(browse);
    }

    public boolean userDataReady() {
	return ((execReq!=null) && super.userDataReady() );
    }

    // Call 'userDataReady()' before using these getters
    public ExecutionRequest getExecutionRequest() {
	return execReq;
    }


    public void actionPerformed(ActionEvent e) {
	Object t= e.getSource(); 
	if(t==agent) { // Validate some entries immediately after input.
	    validateAgentEntry();
	} else if(t== rid) {
	    validateReceiverID();
	} else if(t== browse) {
	    browseFiles();
	} 
    }

    protected void initFields() {
	super.initFields();
	forw.add(hopByHop); // Only execution messages have
	forw.add(broadcast); // these additional forwarding options.
	agent.setText("capsule.FastSendBackAgent"); //ConsistencyAgent VPNAgent PingListenerAgent BandwidthAgent FastestAgent
	validateAgentEntry(); // so that unchanged form is already valid.
	//  anyway it should have a file dialog.
    }


    protected void validateAgentEntry() {
	// Sets the exec request as a side effect.
	execReq=new ExecutionRequest(agent.getText(), null , parseForwarding(),  path, fileExtension, currentAgentNumber++);
	if(!execReq.agentFound()) {
	    execReq=null;
	    agent.setText("AgentNotFound");
	}
    }

    protected ForwardingDescriptor parseForwarding() {
	String entry=forw.getSelectedItem();
	if(entry.equals(hopByHop)) {
	    // Ask towards which destination ip the agent should be hopping.
	    if(!towardsIPdialog.isValid()) { // If the user hasnt selected yet.
		towardsIPdialog.setVisible(true); // Activate the dialog.
		String userInput=towardsIPdialog.getInput();
		int ip=AddressCalculator.parseIPaddress(userInput);
		if(ip!=0) { // If the entered ip is valid
		    towardsIPdialog.setValid();
		    forwarding=new HopByHop(ip);
		    selectSignOnly(); // Forwarding can only be done if enc is signed.
		}
	    }
	    return forwarding;
	}
	if(towardsIPdialog!=null) // If another forw is selected, the ip is invalid,
	    towardsIPdialog.setInvalid(); // so if the selection is changed, the ip can be reentered.
	if(entry.equals(broadcast)) {
	    forwarding=new Broadcast();
	    selectSignOnly();  // Forwarding can only be done if enc is signed.
	    return forwarding;
	    
	}
	return super.parseForwarding();
    }


    protected byte parseEncoding() {
	// Determines the kind of encoding for the protocol object.
	// Since there is only one option, we do ignore the user choice.
	//enc.getSelectedItem();
	String entry=enc.getSelectedItem();
	if(entry.equals(pgp)) return ProtoConsts.FULL_PGP_v263i;
	if(entry.equals(sign)) return ProtoConsts.SIGN_PGP_v263i;
	else return ProtoConsts.PLAIN;
    }

    protected void browseFiles() {
	// Displays a FileDialog and uses the chosen filename to set the 'agent' field.
	// Note: Only good for browsing in the predefined directory, else the classname will probably
	// be wrong.
	fileDialog.setDirectory(path+packageName); //+File.pathSeparator
	fileDialog.show();
	String selected=fileDialog.getFile();
	if (selected==null) return;
	selected=removeExtension(selected);
	selected=packageName+"."+selected; // compose the full classname.
	agent.setText(selected);
	validateAgentEntry();
    }


    protected String removeExtension(String filename) {
	int last=filename.lastIndexOf('.');
	if (last<0) return filename;
	return filename.substring(0,last);
    }
}
