package homeApplication;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

import filter.*;
import utils.*;


public class FilterForm extends Panel implements ActionListener {
    // The user can fill in a filter. Default values are displayed.

    protected TextField src, dst,port, proto, len, nrOp;
    protected Label s,d,sp,p,l,n;
    protected FilterEntryParser parser=new FilterEntryParser();

    protected Filter filter=null;

    public FilterForm() {
	setLayout(new GridLayout(6,2,5,5));
	s=new Label("Source"); d=new Label("Destination");sp=new Label("Port"); p=new Label("Protocol"); l=new Label("Packet len."); n=new Label("Nr. of packets");
	src=new TextField(20);
	dst=new TextField(20);
	port=new TextField(20);
	len=new TextField(20);
	nrOp=new TextField(20);
	proto=new TextField(20);
	initTextFields();

	src.addActionListener(this);dst.addActionListener(this);port.addActionListener(this);len.addActionListener(this);nrOp.addActionListener(this);proto.addActionListener(this);

	add(s);add(src);
	add(d);add(dst);
	add(sp);add(port);
	add(p);add(proto);
	add(l);add(len);
	add(n);add(nrOp);
    }

    public boolean userDataReady() {
	// Checks if a reasonable filter can be parsed from the user entries.
	// Must be called before 'getFilter()'
	if (filter==null) {
	    filter=setFilter();
	    if(filter.isEmpty()) {
		filter=null;
		return false;
	    }
	}
	return true;
    }

    public Filter getFilter() {
	return filter;
    }

    public Filter setFilter() {
	// Get the user entered values out of the form.
	return new Filter(parser.parseAddressRanges(src.getText()),
			  parser.parseAddressRanges(dst.getText()),
			  parser.simplifiedParsing(port.getText()),
			  parser.simplifiedParsing(proto.getText()),
			  parser.parseLength(len.getText()),
			  parser.parseLength(nrOp.getText()));
    }

    public void actionPerformed(ActionEvent e) {
	TextField t=(TextField) e.getSource(); 
	if((t==src) || (t==dst)) {
	    validateAddressEntry(t);
	} else if((t== proto) || (t==port)) {
	    validateSimpleEntry(t);
	} else if((t==len) || (t==nrOp)) {
	    validateLengthEntry(t);
	}
    }


    protected void validateAddressEntry(TextField field) {
	String txt=field.getText();
	if(parser.isWildCard(txt)) return;
	ListOfRanges r=  parser.parseAddressRanges(txt);
	field.setText(parser.printAddressRanges(r));
    }

    protected void validateSimpleEntry(TextField field) {
	ListOfRanges r= parser.simplifiedParsing(field.getText());
	field.setText(parser.printSimplifiedRanges(r));

// 	Wildcard w=parseProtocolRange(proto.getText());
// 	if(w.isEmpty()) proto.setText(""); // or should it be "Error"?
// 	// note: the size should range from 0..2^16-1, which is not ensured here.
    }

    protected void validateLengthEntry(TextField field) {
	String txt=field.getText();
	if(parser.isWildCard(txt)) return;
	Length l=parser.parseLength(txt);
	if (l.meansAny()) field.setText(parser.wildcard());
	else field.setText(l.toString());
    }


    protected void initTextFields() {
	// The default values
	src.setText("130.92.64.15"); // Only localhost traffic. //130.92.70.102
	dst.setText("130.92.65.146"); // 129.194.90.20
	//dst.setText(localHost());
	port.setText("8888");
	proto.setText("17"); //parser.wildcard()

	nrOp.setText("100"); 
	len.setText(parser.wildcard()); // As long as possible.
    }


    protected String localHost() {
	// Looks up the current IP address.
	try {
	    InetAddress a=InetAddress.getLocalHost();
	    return a.getHostAddress();
	} catch (UnknownHostException e) {
	    return "0.0.0.0";
	}
    }



}
