package netgui;

/* Written by Manuel Gnter. All rights reserved, no warranties */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import topology.NodeRecord;


public class NetCanvas extends Component implements MouseListener, MouseMotionListener {
    protected Vector domains=new Vector(); // contains also EndUser
    protected int posCntr=0;

    protected NetComponent selected=null;
    protected boolean dragging=false;

    protected Image offImage; // Vars for double-buffering (avoids glitches).
    protected Graphics offGraphics=null;
    protected Dimension offDimension=new Dimension(0,0);


    public void update(Graphics g) {
	Dimension currentSize=getSize();
	// Create new double buffer if size has changed:
	if ((offGraphics==null) || (!offDimension.equals(currentSize))) {
	    offDimension=currentSize;
	    offImage=createImage(currentSize.width, currentSize.height);
	    offGraphics=offImage.getGraphics();
	}
	
	offGraphics.setColor(getBackground()); // To the old picture.
	offGraphics.fillRect(0,0,currentSize.width, currentSize.height);

	NetComponent d; // Draw all net components       
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    d=(NetComponent) e.nextElement();
	    d.paintOn(offGraphics);
	}
	g.drawImage(offImage,0,0,this);
    }

    public void paint(Graphics g) {
	update(g);
    }

    public void repaint() {
	// Must also be overriden, else there is still flashing. Side effects?
	update(getGraphics());
    }

    public void mouseClicked(MouseEvent me) {
	if(checkIfSelects(me.getX(),me.getY()))
	    repaint();
    }
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent me) {
	if (dragging) {
	    ((RellocNetComponent) selected).move(me.getX(),me.getY());
	    dragging=false;
	}
	repaint();
    }
    public void mouseEntered(MouseEvent e) { }
    public void mouseExited(MouseEvent e) {}
    public void mouseMoved(MouseEvent e) {}
             

    public void mouseDragged(MouseEvent me) {
      if (!dragging)
	  if(checkIfSelects(me.getX(),me.getY())) {
	      dragging=true;
	  }
      if (dragging) {
	  ((RellocNetComponent) selected).move(me.getX(),me.getY());
	  repaint();
      }
    }

    protected boolean checkIfSelects( int x, int y) {
	NetComponent d;
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    d=(NetComponent) e.nextElement();
	    d=d.touching(x,y);
	    if (d!=null){
 		if(selected != null) {   
// 		    addLinkIfRequested(d); // For drawing new links...
 		    selected.deselect();
 		}
		selected=d;
		d.select();
		return true;
	    }
	}	
	return false;
    }

    public Dimension getPreferredSize() { 
	// calculate new size h
 	return new Dimension(600,400);
    }

//     public Dimension getMinimumSize() { 
// 	System.out.println("Min");
//  	return new Dimension(400,400);
//     }

    public void addDomain(Domain d) {
	domains.addElement(d);
    }

    public void addRouter(Domain d, Router r) {
	d.addRouter(r);
    }

    public void addEndUser(EndUser u) {
	domains.addElement(u);
    }
 
    //-- obsolete:
//     public void addNewDomain() {
// 	domains.addElement(new Domain((new Integer(posCntr)).toString(), posCntr++));
// 	repaint();
//     }

//     public void addNewRouter() {
// 	Router router;
// 	if(selected!=null) {
// 	    if(selected instanceof Domain) {
// 		Domain parent=(Domain) selected;
// 		router=new Router(parent, this,  (new Integer(posCntr)).toString());
// 		parent.addRouter(router);
// 		router.attach(new ValueDisplayer(0,1.0));
// 		repaint();
// 	    }
// 	}
//     }

//     public void addNewEndUser() {
// 	domains.addElement(new EndUser(this,posCntr++, (new Integer(posCntr)).toString()));
// 	repaint();
//     }

    // --- Data searchers ---

    public Domain domainOf(String ownerID) {
	// returns null if not found.
	return search(new Domain(ownerID, 1));
    }

    public Router alreadyDisplayed(NodeRecord nr, Domain excludeD) {
	// Checks if the router is already displayed somewhwere. If yes, returns it.
	Object o;
	Domain d;
	Router ret=null;
	Router compare=new Router(nr);
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    o=e.nextElement();
	    if(o instanceof Domain) {
		d= (Domain) o;
		if(d.equals(excludeD)); // Don't treat routers of the same domain.
		else {
		    ret=d.search(compare);
		    if(ret != null) return ret;
		}
	    }
	}
	return ret;
    }

    public EndUser customerAlreadyDisplayed(NodeRecord nr) {
	// Checks if the EndUser is already displayed somewhwere. If yes, returns it.
	Object o;
	EndUser compare=new EndUser(nr);
	return search(compare);
    }

    protected EndUser search(EndUser template) {
	EndUser eu=null;
	Object o;
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    o=e.nextElement();
	    if (o instanceof EndUser) {
		eu=(EndUser) o;
		if (eu.equals(template)) break;
		else eu=null;
	    }
	}
	return eu;
    }

    protected Domain search(Domain template) {
	Domain dom=null;
	Object o;
	for(Enumeration e=domains.elements(); e.hasMoreElements(); ) {
	    o=e.nextElement();
	    if (o instanceof Domain) {
		dom=(Domain) o;
		if (dom.equals(template)) break;
		else dom=null;
	    }
	}
	return dom;
    }

    public String saveResultsOfSelectedRouter(String path) {
	// The results stored in the ValueDisplayer of this router are stored to disk.
	// uses the 1:1 relation router:node
	String fname="", file;
	if(selected instanceof Router) {
	    Router router=(Router) selected;
	    ValueDisplayer vd=router.valueDisplayer();
	    try {
		fname=router.nodeID();
		file=path+fname;
		FileWriter fw=new FileWriter(file); // Store res under the name of the node.
		PrintWriter pw=new PrintWriter(fw);
		vd.plotPrint(pw);
		pw.flush();
		pw.close();
	    } catch(IOException e) {
		System.err.println("NetCanvas: "+e);
		fname="";
	    }
	    
	}
	return fname;
    }

    // ---

    protected void addLinkIfRequested(NetComponent newSel) {
	if(newSel instanceof Router)
	    if (selected instanceof Router) {
		Router a=(Router) newSel;
		Router b=(Router) selected;
		if (a.domain() != b.domain()) {
		    Link l=new Link(a,b);
		    a.addLink(l);
		    b.addLink(l);
		}
	    }
    }    
}
