package netgui;

import java.awt.*;
import java.util.*;

import topology.NodeRecord;

public class Router  extends RellocNetComponent{
    protected static String imgName="/home/mguenter/rvs/Agents/SeparateClientTest/netgui/Icons/routersymb.gif";
    protected static String imgSelName="/home/mguenter/rvs/Agents/SeparateClientTest/netgui/Icons/routersymbSel.gif";
    protected static Image[] imageStore=new Image[4]; // for efficiency. It also
    // is because of the strange behavior with inheritance and static vars.
    protected int imagePos=0;

    protected int xc,yc; // Correction so that x and y are the center of the image.

    protected Component myComp; // The canvas; needed for painting images
    protected Domain domain;
    protected int width=50;

    protected ValueDisplayer valueDisplayer=null; // Maybe this should be attached else where?

    protected Vector links=new Vector();

    protected String nodeID; // Here we assume a 1:1 relation between routers and nodes.

    public Router(Domain d, Component mycomp) {
	// obsolete
	domain=d;
	setInitPos();
	init(mycomp);
    }

    public Router(Domain d, Component mycomp, String nn, String id) {
	netName=nn;
	nodeID=id;
	if(d != null)
	    ownerID=d.ownerID(); 
	domain=d;
	setInitPos();
	init(mycomp);
    }

    Router(NodeRecord nr) {
	// Dummy constructor for use of 'equals()'
	netName=(new Integer(nr.ip)).toString();
	nodeID=nr.key;
	ownerID=nr.organisation;
    }

    public ValueDisplayer valueDisplayer() {
	return valueDisplayer;
    }

    public String nodeID() {
	return nodeID;
    }

    public boolean equals(Router o) {
	System.out.println("Isequal me: "+this+ " and: "+o);
	if(nodeID.equals(o.nodeID)) return true;
	return false;
    }

    protected void init(Component mycomp) {
	myComp=mycomp;
	loadImage(imgName,imgSelName );
    }

    protected void loadImage(String im, String sel) {
	if(imageStore[imagePos]==null) {
	    imageStore[imagePos]=myComp.getToolkit().getImage(im); 
	    imageStore[imagePos+1]=myComp.getToolkit().getImage(sel);
	}
    }

    public void attach(ValueDisplayer vd) {
	valueDisplayer=vd;
	vd.absMove(x,y+30); // Calculate the relative position here.
    }

    public Domain domain() {return domain;}

    public void paintOn(Graphics g) {
	if(xc==0) // if the correction is not yet done
	    calcCorrectionOffset();
	paintLinks(g);
	if (selected) 
	    g.drawImage(imageStore[imagePos+1],x-xc,y-yc,myComp);
	else 
	    g.drawImage(imageStore[imagePos],x-xc,y-yc,myComp);
	paintText(g);
	if(valueDisplayer!=null) // Draw the attached displayer..
	    valueDisplayer.paintOn(g);
    }

    protected void paintText(Graphics g) {
	// Paints the id of the node.
	setFontSize(8, g);
	g.setColor(foreground2);
	g.drawString(nodeID,x-xc,y-yc-1); // Draw in the left upper corner.
    }


    NetComponent touching(int x, int y) {
	if ((this.x-xc<=x) && (this.x+xc >= x) && (this.y-yc<=y) && (this.y+yc >= y))
	    return this;
	return null;
    }

    void move(int x, int y) {
	// Ensures that the router stays at the border of the parent net.
	Dimension parentpos=domain.position();
	int px,py,dx,dy,rad;
	rad= domain.radius();
	px= parentpos.width;
	py= parentpos.height;
	dx=x-px;
	dy=y-py;
	double dist=Math.sqrt(dx*dx+dy*dy);
	if (dist==0) return;
	float stretch=rad/((float) dist);
	float nx,ny;
	nx=px+dx*stretch;
	ny=py+dy*stretch;

	absMove(Math.round(nx), Math.round(ny));
    }

    void addLink(Link l) {
	// A duplicate check would be nice...
	links.addElement(l);
    }

    public void addBidirectLinkTo(Router b) {
	    Link link=new Link(this, b); // Add the access link..
	    addLink(link);
	    b.addLink(link);
    }

    protected void absMove(int x, int y) {
	if(valueDisplayer!=null)
	    valueDisplayer.relMove(x-this.x,y-this.y);
	super.absMove(x,y);
    }

    protected void relMove(int x, int y) {
	if(valueDisplayer!=null)
	    valueDisplayer.relMove(x,y);
	super.relMove(x,y);
    }

    protected void  paintLinks(Graphics g) {
	Link l;
	for(Enumeration e=links.elements(); e.hasMoreElements(); ) {
	    l=(Link) e.nextElement();
	    if(l.routerA()==this) l.paintOn(g); // don't draw the links twice.
	}	
    }

    protected void calcCorrectionOffset() {
	//x,y should be the center of the image, not the left-upper corner
	xc=(int) imageStore[imagePos].getWidth(myComp)/2; 
	yc=(int) imageStore[imagePos].getHeight(myComp)/2; //...assuming that symbSel is as large as symb!
    }

    protected void setInitPos() {
	// Set position at 3 o'clock on the border of the parent network.
	Dimension parentpos=domain.position();
	x=parentpos.width+domain.radius(); 
	y=parentpos.height;
    }

}
