package netgui;

import java.awt.*;
import java.util.*;
import java.io.*;

import clientserver.NumericResults;
import homeApplication.CallBackDisplay; // to see if recording is on/off.


public class ValueDisplayer  extends RellocNetComponent{
    // Displays a little color-coded bar-diagram.
    // Made for displaying clientserver.NumericalResults
    // Currently attached to netgui.Routers.

    protected static final Color errorCol=Color.red; 
    protected static final Color warningCol=Color.yellow; 
    protected static final Color okCol=Color.green; 

    protected static final int width=6; // e.g. the width of the bar. 
    protected static final int spacing=12; //  Space between elements.
    protected static final int lineSpacing=4; //Space between text and bar.


    protected int repSize=64; // Lenght of the bar in pixels.
 
    protected String unit="";
    protected double minVal=0; //The range of displayable values.
    protected double maxVal=1;
    protected double warningThres; 
    protected double errorThres; 

    protected NumericResults currentRes=new NumericResults(0,0.0);
    protected Vector resultStore;

    protected String error=null;

   public ValueDisplayer(ValueDisplayer d) {
       // Copy constructor
       unit=d.unit;warningThres=d.warningThres; errorThres=d.errorThres;
       repSize=d.repSize;				
       setExtremes(d.minVal, d.maxVal);
   }

    public ValueDisplayer(double min, double max) {
	setExtremes(min,max);
	defaultThresInit();
    }

    public ValueDisplayer(int repSize, double min, double max) {
       this.repSize=repSize;
       setExtremes(min,max);
       defaultThresInit();
    }

    public  ValueDisplayer(String unit, double min, double max) {
       setExtremes(min,max);
       defaultThresInit();
       setUnit(unit);
    }

    protected void defaultThresInit() {
	warningThres=0.5*maxVal;
	errorThres=0.75*maxVal; 
    }
    
    public void setResult(NumericResults r) {
	currentRes=r;
	if (currentRes.result>maxVal) {
	    System.err.println("Value display warning: result>maxVal.");
	    currentRes.result=maxVal;
	}
	if (currentRes.result<minVal) {
	    System.err.println("Value display warning: result<minVal.");
	    currentRes.result=minVal;
	}
	storeResult(r); // If recording is on, store the sub-results.
    }

    public void setUnit(String u) {unit=u;}
    public void setExtremes(double min, double max) {
	if(min>max) {
	    System.err.println("Value display error: min>max.");
	}
	minVal=min;
	maxVal=max;

    }
    public void setThres(double warning, double error) {
	if(error<warning ) {
	    System.err.println("Warning: strange threshold setting.");
	}
	errorThres=error;
	warningThres=warning;
    }

    public void setError(String error) {
	this.error=error;
    }

    public void paintOn(Graphics g) {
	if(error!=null)
	    paintError(g);
	else if(currentRes.alarm)
	    paintAlarm(g);
	else if(currentRes.isLast())
	    paintFinished(g);
	else normalPaintOnGraphics(g);
    
    }


    protected void paintAlarm(Graphics g){
	paintSpecialText("ALARM" , g);
    }

    protected void paintError(Graphics g){
	paintSpecialText(error , g);
    }

    protected void paintFinished(Graphics g){
	paintSpecialText("Connection closed.\n"+currentRes.toString() , g);
    }

    protected void normalPaintOnGraphics(Graphics g){
	paintValueBar(g);
	paintText(g);
    }

    protected void paintValueBar(Graphics g) {
	double val=currentRes.result;
	int sx=x; 
	int sy=y+spacing;
	paintGrid(g,sx,sy);
	g.setColor(okCol);
	int scaledVal=scaleVal(val);
	int scaledThres;
	g.fillRect(sx, sy, scaleVal(val), width);
	if(val>warningThres) {
	    g.setColor(warningCol);
	    scaledThres=scaleVal(warningThres);
	    g.fillRect(sx+scaledThres, sy, scaledVal-scaledThres, width);
	}
	if(val>errorThres) {
	    g.setColor(errorCol);
	    scaledThres=scaleVal(errorThres);
	    g.fillRect(sx+scaledThres, sy, scaledVal-scaledThres, width);
	}
	
	g.setColor(foreground);
    }

    protected void paintGrid(Graphics g,int x,int y) {
	int[] xs=new int[4];
	xs[0]=x; xs[1]=x; xs[2]=x+repSize; xs[3]=x+repSize;
	int[] ys=new int[4];
	ys[0]=y-2; ys[1]=y-1; ys[2]=y-1; ys[3]=y-2;
	g.drawPolyline(xs,ys,4);
    }

    protected void paintSpecialText(String text ,Graphics g) {
	setFont(g);
	g.setColor(errorCol);
	super.setFontSize(14, g);
	g.drawString(text,x,y);
    }

    protected void paintText(Graphics g) {
	setFont(g);
	g.setColor(selectedColor);
	int ypos=y+spacing-lineSpacing;
	g.drawString((new Date(currentRes.time/1000)).toString(),x,ypos);
	ypos+=2*spacing;
	g.drawString((new Double(currentRes.result)).toString()+" "+unit,x,ypos);
    }

    protected int scaleVal(double v) {
	// Scales a value to the screen representation.
	double sc=(v-minVal)/maxVal;
	return (int) Math.round(sc*repSize);
    }

    protected void setFont(Graphics g) {
	setFontSize(spacing-lineSpacing, g);
    }

    NetComponent touching(int x, int y) {
	return null;
    }

    public String toString() {
	return super.toString()+" "+minVal+","+maxVal+unit;
    }

    public void plotPrint(PrintWriter ps) throws IOException {
	// for printing the store
	if(resultStore==null) return;
	NumericResults nr;
	for(Enumeration e=resultStore.elements(); e.hasMoreElements(); ) {
	    nr=(NumericResults) e.nextElement();
	    nr.plotPrint(ps);
	}
    }

    protected void storeResult(NumericResults res) {
	boolean on=CallBackDisplay.record;
	if(on) {
	    if(resultStore==null) resultStore=new Vector();
	    resultStore.addElement(res); // Add the results submitted.
	} else {
	    if(resultStore!=null) resultStore=null; // Empty/reset the store.
	}
    }

}
