package stat;

public abstract class DiscreteDistribution {
    protected int nrOfCategories=-1;

    public int getNrOfCategories() {
	return nrOfCategories;
    }

    public double getProbabilityFor(int category) {
	if(isNotOk(category)) {
	    System.err.println("Illegal categroy index: "+category+"\nReturning probability 0.");
	    return 0;
	}
	return internalGetProbFor(category);
    }

    protected abstract double internalGetProbFor(int category);


    // Sanity checks

    public boolean isNotOk(int category) {
	return (category<0) || (category>=nrOfCategories);
    }

    public boolean isPositive() {
	for (int i=0; i<nrOfCategories; i++ ) {
	    if (internalGetProbFor(i)<0.0) return false;
	}
	return true;
    }

    public boolean sumsUpCorrectlyTo(double correct) {
       return (Math.abs(sum()-correct)<1e-8);
    }

    public boolean isSane() {
	return isPositive() && sumsUpCorrectlyTo(1.0);
    }


    protected double sum() {
       double sum=0.0;
       for (int i=0; i<nrOfCategories; i++ ) {
	   sum+=internalGetProbFor(i);
       }
       return sum;
    }

    // --- Other helpers
    protected double min() {
	double h, min=internalGetProbFor(0);
	for (int i=1; i<nrOfCategories; i++ ) {
	   h=internalGetProbFor(i);
	   if (h<min) min=h;
	}	return min;
    }

    // 


    // Testing- I/O

    public String toString() {
	StringBuffer buff=new StringBuffer();
	buff.append("DiscreteDistribution with ");
	buff.append(nrOfCategories);
	buff.append(" categories.\n");

	for (int i=0; i<nrOfCategories; i++ ) {
	    buff.append(i);
	    buff.append('\t');
	    buff.append(internalGetProbFor(i));
	    buff.append("\n");
	}
	return new String(buff);
    }
}
