package utils;

import filter.*;
import node.*; // for the IPPacket def.
import java.util.*;
import java.net.*;

public class AddressCalculator {
    public static int[] toArray(int a) {
	// Hacking around with the 2-complement...
	int ret[]=new int[4];
	long h, toPos=1;
	toPos<<=31;
	long l=(a<0)? toPos+a: a;
	
	for (int i=3; i>0; i--) {
	    h=l%256;
	    l=l>>8;
	    ret[i]=(int) h;
	}
	h=l%256;
	if (a<0) h+=128;
	ret[0]=(int) h;
	return ret;
    }

    public static String toString(int a) {
	return toString(a, 4);
    }

    protected static String toString(int a, int pos) {
	if(pos<1) return "Error";
	StringBuffer buf=new StringBuffer(16);
	int[] decs=toArray(a);
	for (int i=0; i<pos-1; i++) {
	    buf.append(decs[i]);
	    buf.append('.');
	}
	buf.append(decs[pos-1]);
	return new String(buf);
    }

    public static String toDotString(Range r) {
	// write a range in 130.65 form. This is of course not possible
	// for all ranges.
	if(r.isMaximal()) return FilterEntryParser.WILDCARD;
	int i=0;
	int l=r.low();
	int h=l;
	int u=r.up();
	for(i=0;i<4;i++) {
	    if (((l & 0xff)!=0) || ((u & 0xff)!=255)) break;
	    l=l>>8;
	    u=u>>8;
	}

	if(u != l) System.err.println("Warning: range "+r+" does not describe a subnet");
	i=4-i;
	//int[] decs=toArray(l);
	return toString(h,i);
    }


    public static int parseIPaddress(String s) {
	return toInt(parseBytePos(s, 4));
    }

    public static int toInt(int[] d) {
	// For ints in the range 0..255
	int ret=0;
	for (int i=0; i<4; i++) {
	    if ((d[i]>255) || (d[i]<0)) {
		System.err.println("Not a byte value: "+d[i]);
		return 0;
	    }
	    ret=ret<<8;
	    ret+=d[i];
	}
	return ret;
    }

    public static int toInt(byte[] bytes, int pos, int len) {
	return toInt(bytes, pos, len, true);
    }

    public static int toBigEndInt(byte[] bytes, int pos, int len) {
	return toInt(bytes, pos, len, false);
    }

//     public static int toInt(byte[] bytes, int pos, int len) {
// 	// Helper for IPPacket (little-endian)
// 	int ret=0;
// 	int up=pos+len;
// 	int h;
// 	for (int i=pos; i<up; i++) {
// 	    h=bytes[i];
// 	    if (h<0) h+=256;
// 	    ret=ret<<8;
// 	    ret+=h;
// 	}
// 	return ret;
//     }  // Old and ok method.

    public static int toInt(byte[] bytes, int pos, int len, boolean littleEnd) {
	// Helper for IPPacket (little-endian)
	int ret=0,low,up,inc, h;
	if(littleEnd) {
	    low=pos;
	    up=pos+len;
	    inc=1;
	} else {
	    low=pos+len-1;
	    up=pos-1;
	    inc=-1;
	}
	for (int i=low; i!=up; i+=inc) {
	    h=bytes[i];
	    if (h<0) h+=256;
	    ret=ret<<8;
	    ret+=h;
	}
	return ret;
    }

    
    public static Range parseRange(String s) {
	int[] pars=parseBytePos(s, 1);
	int[] low=new int[4];
	int[] up=new int[4];
	int val=0;
	for(int i=0; i<4; i++) {
	    val=pars[i];
	    if (val<0) { // if not parsed
		low[i]=0;
		up[i]=255;
	    } else {
		low[i]=up[i]=val;
	    }
	}
	return new Range(toInt(low),toInt(up));

    }


    public static String dnsToIP(String dns) {
	InetAddress ina=null;
	try {
	    ina=InetAddress.getByName(dns);
	} catch(UnknownHostException e) {
	    System.err.println("Warning: DNS lookup failed!");
	    return "";
	}
	return ina.getHostAddress();
    } 

    protected static int[] parseBytePos(String s, int pos) {
	// Parse the form 130.92.65.25. Parse at least pos bytes
	int decs[]=new int[4];
	for(int i=0; i<4; i++){decs[i]=-1;} // To see how many positions were parsed.
	StringTokenizer parser = new StringTokenizer(s,".",false);
	//for (int i=0; i<pos; i++) {
	int i=0;
	boolean cont=true;
	while(parser.hasMoreTokens() && cont) {
	    if (!parser.hasMoreTokens() && (i<pos)) {
		System.err.println("Not enough decimals given or wrong formated.");
		return decs;
	    }
	    try {
		decs[i]=Integer.parseInt(parser.nextToken());
	    }
	    catch(NumberFormatException e) {
		System.err.println("Your IP address is badly formated.\n"+e);
		cont=false;
	    }
	    i++;
	    if(i>4) {
		System.err.println("This is too long for an IP address!");
		cont=false;
	    }
	    
	}
	return decs;
    }


    public static void main(String[] args) {
// 		System.out.println(toString(-1408171930));
// 		System.out.println(toString(-1408171931));

// 	System.out.println(toString(parseIPaddress("130.92.65.25")));
// 	System.out.println(toString(parseIPaddress("1.255.1.255")));
// 	System.out.println(toString(parseIPaddress("256.255.1.255")));
// 	System.out.println(parseRange(".92.65"));
// 	System.out.println(parseRange("1.92"));
// 	System.out.println(parseRange("2."));
// 	System.out.println(parseRange("2"));
// 	System.out.println();
// 	System.out.println(toDotString(parseRange("250.92.65")));
// 	System.out.println(toDotString(parseRange("130.92")));
// 	System.out.println(toDotString(parseRange("255.")));
// 	System.out.println(toDotString(parseRange("2.2.2.0")));
// 	System.out.println();

 	System.out.println(parseRange("130.92"));
 	System.out.println(parseRange("130.93.66"));


    }

}
