package utils;

import node.IPPacket;
import java.util.Date;

public class PacketPrinter {
    protected StringBuffer buf;

    public String toString(IPPacket p) {
	buf=new StringBuffer(1024);
	printHeaderInfo(p);
	printData(p);
	return new String(buf);
    }

    public String headerInfo(IPPacket p) {
	buf=new StringBuffer(1024);
	printHeaderInfo(p);
	return new String(buf);
    }

    public String data(IPPacket p) {
	buf=new StringBuffer(1024);
	printData(p);
	return new String(buf);
    }

    public String data(byte[] raw) {
	buf=new StringBuffer(1024);
	printData(raw, 6);
	return new String(buf);
    }

    protected void printExactDate(IPPacket p) {
	// Use the timestamp to create the readable date.
	buf.append(new Long(p.timeStamp/1000)); // Latency tests
	buf.append(' ');
	buf.append(new Date(p.timeStamp/1000));
	buf.append(' ');
	buf.append(p.timeStamp%1000000);
	buf.append(" mc.s.");
    }


    protected void printHeaderInfo(IPPacket p) {
    	buf.append('\n');
	printExactDate(p);
	buf.append(" Packet ");
	buf.append("\n\t");
	buf.append(" from: ");
	buf.append(AddressCalculator.toString(p.source()));
	buf.append(" to: ");
	buf.append(AddressCalculator.toString(p.destination()));
	buf.append("\n\t");
	buf.append(" proto: ");
	buf.append(p.proto());
	buf.append(" len: ");
	buf.append(p.len());
	buf.append('\n');
    }

    protected void printData(IPPacket p) {
	printData(p.bytes, 0);
    }

    protected void printData(byte[] raw, int colOff) {
	int col=colOff; // start like in the tcpdump file (20 bytes heade omitted)
	buf.append("\t\t ");
	for(int i=0;i<col;i++){buf.append("     " );}
	for (int i=0; i<raw.length;) {
	    if ( col==8) {
		buf.append("\n\t\t ");
		col=0;
	    }
	    printTwoBytes(raw, i);
	    col++;
	    i+=2;
	}

    }

    protected void printTwoBytes(byte[] d, int i){
	buf.append(' ');
	printByte(d[i]);
	i++;
	if(d.length>i) 
	    printByte(d[i]);
    }

    protected void printByte(byte b) {
	int h=b;
	if (b<0) h=256+b;
	int l=h%16;
	buf.append(hexChar(h>>4));// upper
	buf.append(hexChar(l)); // lower 4 bits
    }

    protected char hexChar(int c) {
	if ((c<0) || (c>15)) {
	    System.err.println("No hex value: "+c);
	    return 0;
	}
	if (c>9) return (char) (c+87);
	return (char) (c+48);
    }
}
