package utils;

import java.net.*;
import java.io.*;

public class PrimitiveTypeParser {
    // Parses Strings into primitive types, does some basic sanity checks.
    // Stops the application upon exception.

    public long toLong(String token) {
	long ret=0;
	try {
	    ret=Long.parseLong(token);
	} catch(NumberFormatException e) {
	    onError(token,"long");
	}
	return ret;    } 

    public int toInt(String token) {
	int ret=0;
	try {
	    ret=Integer.parseInt(token);
	} catch(NumberFormatException e) {
	    onError(token,"int");
	}
	return ret;
    } 

    public double toDouble(String token) {
	Double ret=new Double(0);
	try {
	    ret=Double.valueOf(token);
	} catch(NumberFormatException e) {
	    onError(token,"double");
	}
	return ret.doubleValue();
    } 
    public boolean toBoolean(String token) {
	Boolean ret;
	ret=Boolean.valueOf(token); // Warning: if not 'true' is parsed,
	// then false is returned ('ogelbagogel' parses to false).
	return ret.booleanValue();
    } 

    public String toDNS(String token) {
	try {
	    InetAddress.getByName(token);
	} catch(UnknownHostException e) {
	    onError(token,"valid DNS address");
	}
	return token;
    } 
    public String toPath(String token) {
	// Does only a check.
	File dir=new File(token);
	if(!dir.isDirectory()) {
	    System.out.println("Error: directory does not exist:");
	    onError(token,"valid directory");
	}
	return token;
    } 

    public int toPort(String token) {
	int ret=toInt(token);
	if(ret<0 || ret>65535) {
	    System.out.println("Error: port out of range:");
	    onError(token,"valid port number");
	}
	return ret;
    } 

    protected void onError(String token, String expected) {
	System.err.println("Error while parsing configuration!");
	System.err.println("Expecting: "+expected+" but got: "+token);
	System.exit(9); // If the configuration file is buggy, stop the app.
    }

}
